%=================================================================%
% Calculates the Fourier coefficients of the function f           %
% up to the order K. f is a string containing the name            %
% of the m-file, where the function is implemented.               %
% This function performs the complex Fourier analysis,i.e. we     %
% take {1, e^( 2 i pi k (x-alfa)/L), e^(-2 i pi k (x-alfa)/L) }   %
% as L^2 Hilbert-space basis. L=beta-alfa                	  %			
%=================================================================%

function [fc1, fcEP, fcEM] = complexFourier(K,f)
addpath ./functions

global alfa;       %left  boundary
global beta;       %right boundary
fcEP = zeros(1,K); %array storing Fourier coefficients of e^( 2 i pi k (x-alfa)/L)
fcEM = zeros(1,K); %array storing Fourier coefficients of e^(-2 i pi k (x-alfa)/L)

  integrand = inline(strcat(f,'(x).*normedcon(x)'));
  fc1 = quad(integrand,alfa,beta);

  %note that for the scalar product complex-conjugation is taken
  %into account:  conj(normedEP) = normedEM; conj(normedEM) = normedEP

  %normedEP
  for k=1:K,
    integrand = inline(strcat(f,'(x).*normedEM(',int2str(k),',x)'));
    fcEP(k) = quad(integrand,alfa,beta);
  end;

  %normedEM
  for k=1:K,
    integrand = inline(strcat(f,'(x).*normedEP(',int2str(k),',x)'));
    fcEM(k) = quad(integrand,alfa,beta);
  end;


%--------------------------------------------------------------------
% just for testing:
% visualization of function and approximated Fourier synthesis
%--------------------------------------------------------------------

if(0)
  M     =  100;
  dx    =  (beta-alfa)/M;
  x     =  [alfa:dx:beta-dx];
  ue    =  zeros(1,M);
  u     =  zeros(1,M);

  u     =  fc1*normedcon(x);
  u     =  u + fcEP*normedEP([1:K],x);
  u     =  u + fcEM*normedEM([1:K],x);

  fun   = inline(strcat(f,'(x)'));
  ue    = fun(x);

  hold on;
  plot([x beta],[ue ue(1)],'b-');
  plot([x beta],[u  u(1) ],'r-');
end;%if