%=================================================================
% Christmas Tutorial to 'Numerical Methods for PDEs', WS 01/02
% Lecture given by Dr.C.Coclici, assisted by M.Rheinlnder
%=================================================================
%
%
%=================================================================
clear all;
addpath ./functions
global alfa;
global beta;

%-----------------------------------------------------------------
% initial condition and initialization of parameters
%-----------------------------------------------------------------
u_0   = 'jump'; %the string must be the name of a matlab function
u0    = inline(strcat(u_0,'(x)'));
%-----------------------------------------------------------------

alfa  = -1;                %right interval boundary
beta  =  1;                %left  interval boundary

K     = 100;               %number of Fourier terms
M     = 100;               %number of subintervals
dx    = (beta-alfa)/M;     %spatial increment
x     = [alfa:dx:beta-dx]; %grid nodes
y     = zeros(1,M);        %shifted grid nodes
u     = zeros(1,M);        %solution

xe    = [alfa:dx/10:beta-dx/10]; %finer grid; 10 times finer
ye    = zeros(1,10*M);     %shifted finer grid
ue    = zeros(1,10*M);     %exact solution
ua    = zeros(1,10*M);     %solutuion to improved modified equation
um    = zeros(1,10*M);     %solution to modified equation
K     = 10;                %number of terms in Fourier expansion

t     = 0;                         %time
n     = 0;                         %iteration
a     = 0.8;                       %advection speed
cn    = 0.7;                       %Courant number cn = a*dt/dx
dt    = dx*cn/a;                   %time increment
S     = (dt^2*a^3 - dx^2*a)/6;     %coefficient before u_xxx   (dispersion term)
D     = (a^2*dx^2*dt-a^4*dt^3)/24; %coefficient before u_xxxx  (diffusion  term)

%----------------------------------------------------
%calculation of the Lax-Wendroff scheme matrix
%----------------------------------------------------
  P = zeros(M,M);        %scheme matrix
  P(1,M) = 0.5*(cn+cn^2);
  P(1,1) = 1-cn^2;
  P(1,2) = 0.5*(cn^2-cn);
  for j=2:M-1,
    P(j,j-1) = 0.5*(cn+cn^2);
    P(j,j)   = 1-cn^2;
    P(j,j+1) = 0.5*(cn^2-cn);
  end;
  P(M,M-1) = 0.5*(cn+cn^2);
  P(M,M)   = 1-cn^2;
  P(M,1) = 0.5*(cn^2-cn);
  Q = inv(P);            %inverse scheme matrix

%----------------------------------------------------
% coefficients of the Fourier series
%----------------------------------------------------
  disp('calculate Fourier coefficents');
  [fc1,fcEP,fcEM] = complexFourier(K,u_0);

%----------------------------------------------------
% while-loop
%----------------------------------------------------
  %initialization
  u     = u0(x);
  ue    = u0(xe);
  um    = u0(xe);
  ua    = u0(xe);

  while(1)
    figure(1); clf; hold on;
    plot([x beta], [u  u(1) ],'r-');
    plot([xe beta],[ue ue(1)],'y-');
    plot([xe beta],[um um(1)],'b-');
    plot([xe beta],[ua ua(1)],'g-');
    title('The Modified Equation for the Lax-Wendroff Scheme');
    text(alfa+0.03*(beta-alfa),1.45,['iteration n= ' int2str(n) ]);
    legend('LW numeric solution','exact sol. of advection eqn.','exact sol. of modified eqn.','exact sol. of improved modified eqn.');
    axis([alfa beta -0.5 1.5]);
    hold off;

    N = input('number of performing time-steps ','s');
    if(isempty(N)==1) N=1;
    elseif(N=='q') break; end;
    N = numeric(N);
    t = t + N*dt;  n = n+N;
    ye = alfa + mod((xe-alfa) - a*t, beta-alfa);
    ue = u0(ye);
    %time-factor: note that i is the imaginary unit
      atfEP = exp(-i*S*t*(2*pi*[1:K]/(beta-alfa)).^3).* exp(-D*t*(2*pi*[1:K]/(beta-alfa)).^4);
      atfEM = exp( i*S*t*(2*pi*[1:K]/(beta-alfa)).^3).* exp(-D*t*(2*pi*[1:K]/(beta-alfa)).^4);
       tfEP = exp(-i*S*t*(2*pi*[1:K]/(beta-alfa)).^3);
       tfEM = exp( i*S*t*(2*pi*[1:K]/(beta-alfa)).^3);
    um     = fc1 *  normedcon(ye);
    um     = um  + (fcEP.*tfEP)*normedEP([1:K],ye);
    um     = um  + (fcEM.*tfEM)*normedEM([1:K],ye);
    um     = real(um);
    ua     = fc1 *  normedcon(ye);
    ua     = ua  + (fcEP.*atfEP)*normedEP([1:K],ye);
    ua     = ua  + (fcEM.*atfEM)*normedEM([1:K],ye);
    ua     = real(ua);
    if(N>0)     u  = (P^N * u')';
    elseif(N<0) u  = (Q^(-N) * u')';
    end;%if
  end; %while
